#include "IZ_app.h"

void IZ_AppHandleSignal(i32 _signal) {
	global_app->server.ws.interrupted = true;
	IZ_WSServerCancelService(&global_app->server);
}

// TODO move to each subsystem
// TODO unify loading of config from cmdline and config file
void IZ_AppLoadConfig(IZ_App *app, u8 argc, const char **argv) {
	memcpy_s(app, sizeof(IZ_App), &IZ_APP_DEFAULT_STATE, sizeof(IZ_App));

	const char *cmdline_buffer;
	if ((cmdline_buffer = lws_cmdline_option(argc, argv, "-d"))) {
		app->config.log_level = atoi(cmdline_buffer);
	}

	if ((cmdline_buffer = lws_cmdline_option(argc, argv, "-p"))) {
		app->config.port = atoi(cmdline_buffer);
	}
}

IZ_ProcedureResult IZ_AppInitialize(IZ_App *app, u8 argc, const char **argv) {
	signal(SIGINT, IZ_AppHandleSignal);
	signal(9, IZ_AppHandleSignal);
	signal(SIGTERM, IZ_AppHandleSignal);

	IZ_AppLoadConfig(app, argc, argv);
	IZ_LogInterceptWSMessages(app->config.log_level);
	if (IZ_WSServerInitialize(&app->server, (IZ_WSServerInitializeParams) {
		.protocol = NULL,
		.port = app->config.port,
		.default_filename = NULL,
		.origin = NULL,
	})) {
		return -1;
	}

	global_app = app;
	return 0;
}

IZ_ProcedureResult IZ_AppRun(IZ_App *app, u8 argc, const char **argv) {
	if (IZ_AppInitialize(app, argc, argv)) {
		return -1;
	}

	i32 result = 0;
	while (true) {
		if (IZ_WSServerHandle(&app->server)) {
			result = -1;
			break;
		}

		if (app->server.ws.interrupted) {
			break;
		}
	}

	IZ_WSServerTeardown(&app->server);
	lwsl_user("Server closed. Bye!\n");
	return result;
}
